/*  This file is part of epubgrep.
 *  Copyright © 2021 tastytea <tastytea@tastytea.de>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as published by
 *  the Free Software Foundation, version 3.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef EPUBGREP_SEARCH_HPP
#define EPUBGREP_SEARCH_HPP

#include "fs-compat.hpp"
#include "options.hpp"

#include <boost/regex.hpp>

#include <cstddef>
#include <cstdint>
#include <map>
#include <string>
#include <string_view>
#include <utility>
#include <vector>

namespace epubgrep::search
{

using match_context = std::pair<std::string, std::string>;

struct match
{
    fs::path filepath_epub;      //!< File path of the EPUB.
    std::string text;            //!< Matched string.
    match_context context;       //!< The context around the match.
    std::string filepath_inside; //!< The file path of the matched line.
    std::string headline;        //!< The last headline, if available.
    std::string page;            //!< The page number, if available.
    std::string language;        //!< Match language.
} __attribute__((aligned(128)));

struct settings
{
    options::regex_kind regex{options::regex_kind::basic};
    bool grep_like{false};
    bool ignore_case{false};
    bool raw{false};
    std::uint64_t context{0};
} __attribute__((aligned(16)));

struct file_in_epub
{
    std::string text;
    std::map<size_t, std::string> headlines;
    std::map<size_t, std::string> pages;
} __attribute__((aligned(128)));

//! Search file, return matches.
[[nodiscard]] std::vector<match> search(const fs::path &filepath,
                                        std::string_view regex,
                                        const settings &opts);

//! Return words before and after the match.
[[nodiscard]] match_context
context(const boost::match_results<std::string::const_iterator> &match,
        std::uint64_t words);

} // namespace epubgrep::search

#endif // EPUBGREP_SEARCH_HPP
